//****************************************************************************
// MODULE NAME: Reports
//
// SPECIFICATION FILE: Reports.h
//
// PROGRAMMING CONSIDERATIONS:  None. yet
//
//***************************************************************************

#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <iomanip>
#include <unistd.h>
#include <sys/wait.h>
#include <math.h>
#include "Reports.h"
#include "SQL_UTIL.h"


void Reports::new_loans_report( void ) {

  int line = 0, a;
  char buffer[400];
  char New_Loan_Filename[] = "NewLoansReport.txt";
  FILE *report;

  // initialize borrower array
  for( a = 0; a < 4; a++ ) {
    Borrower_ID[a] = -1;
    Borrower_Name[a][0] = '\0';
  }

  // open New Loans Report
  if( !(report = fopen( New_Loan_Filename, "r+" ) ) ) {
    // report error File Not Found
    printf( "Error: Could not open %s", New_Loan_Filename );
    exit( 1 );
  }
  quick_connect(); // temp method, workaround for no login box functionality

  // Start new transaction 
  begin_transaction();

  // Parse file
  while( !feof( report ) ) {

    fgets( buffer, sizeof(buffer), report );
    line++;
    parse_line( buffer );
    
    if( verified() ) {
      // Verify New Loan with user??
      // Insert Loan into DB
      add_loan( Loan_ID, Lender_ID, Loan_Type, Loan_Status, Property_ID,
                Original_Value, ARM_Principal, Rate_Type, Interest_Rate,
                Term, MM, DD, YYYY, FMCO_Purch                           );


      // Add Borrowers, then Reset Variables
      for( a = 0; a < 4; a ++ ) {
        if( Borrower_ID[a] >= 0 && Borrower_Name[a][0] != '\0' ) {
          add_borrower( Borrower_ID[a], Borrower_Name[a] );
          add_b_have_l( Borrower_ID[a], Loan_ID );
        }
        Borrower_ID[a] = -1;
        Borrower_Name[a][0] = '\0';
      }
    }
  }
  // Commits transaction and disconnects from the database
  commit_transaction();

  // Close File
  fclose( report );
}

bool Reports::verified( void ) {
  if( (Loan_ID < 0) || (Lender_ID < 0) || (FMCO_Purch < 0) || 
      (Original_Value < 0) || (Record_Type < 0) || (Interest_Rate < 0) ||
      (Rate_Type < 0) || (ARM_Principal < 0) || (MM < 0) || (DD < 0) ||
      (YYYY < 0) || (Loan_Type < 0) || (Property_ID < 0) || 
      (Loan_Status < 0) || (Borrower_ID[0] < 0 ) || 
      (Borrower_Name[0] == NULL )                                        )
  {
    return false;
  }
  return true;
}
  
void Reports::loan_status_report( void ) {
 
  char delim[] = ":\n";
  char *token;
  int line = 0, a;
  char buffer[400];
  char Date[12];
  char Loan_Status_Filename[] = "LoanStatusReport.txt";
  FILE *report;

  if( !( report = fopen( Loan_Status_Filename, "r+" ) ) ) {
    // Error File not Found
    printf( "Error: could not open %s", Loan_Status_Filename );
    exit( 1 );
  }
  quick_connect();
  begin_transaction();
  while( !feof( report ) ) {
    fgets( buffer, sizeof(buffer), report );
    line++;
    parse_line( buffer );
    if( verified() ) {
      // Update Loan with New Info
      update_loan( Loan_ID, Lender_ID, Loan_Type, Loan_Status, Property_ID,
                Original_Value, ARM_Principal, Rate_Type, Interest_Rate,
                Term, MM, DD, YYYY, FMCO_Purch                           );


      // Update existing Borrower and NOT Purchased loan relationship
      // Then Reset Variables
      for( a = 0; a < 4; a ++ ) {
        if( Borrower_ID[a] >= 0 && Borrower_Name[a][0] != '\0' 
            && !FMCO_Purch                                     ) {
          update_b_have_l( Borrower_ID[a], Loan_ID );
        }
        Borrower_ID[a] = -1;
        Borrower_Name[a][0] = '\0';
      }
    }
  }
  commit_transaction();
  //Close File
  fclose( report );
}


int Reports::get_int( char *token, int min, int max ) {

  int ret;

  if( token ) {
    sscanf( token, "%d", &ret );
    if( ret <= max && ret >= min ) {
      return ret;
    }
  }
  // report error
  return -1;
}


double Reports::get_double( char *token, double min, double max ) {

  double ret;

  if( token ) {
    sscanf( token, "%lf", &ret );
    if( ret <= max && ret >= min ) {
      return ret;
    }
  }
  // report error
  return -1;
}

float Reports::get_float( char *token ) {

  float ret;
  int truncate_garbage;

  if( token ) {
    sscanf( token, "%f", &ret );
    if( ret <= 99.999 && ret >= 0.001 ) {
      truncate_garbage = (int)(1000.00 * ret);
      ret = truncate_garbage / 1000.00;
      return ret;
    }
  }
  // report error
  return -1;
}

char* Reports::get_name( char *token ) {

  int len;

  if( token ) {
    len = strlen( token );
    if( len <= 64 && len >= 5 ) {
      return token;
    }
  }
  // report error
  return NULL;
}

//void main( void ) {
  // Method for compiling purposes only
//}

void Reports::parse_line( char *buffer ) {

  char delim[] = ":\n";
  char *token, *temp_name;
  int a;
  char *temp_date;

  token = strtok( buffer, delim );
  Record_Type = get_int( token, 0, 99 );

  token = strtok( NULL, delim );
  Loan_ID = get_double( token, 0, (pow( 10, 11 ) - 1) );

  token = strtok( NULL, delim );
  Lender_ID = get_double( token, 0, (pow( 10, 9 ) - 1) );

  token = strtok( NULL, delim );
  FMCO_Purch = get_int( token, 0, 1 );

  token = strtok( NULL, delim );
  Original_Value = get_double( token, 1000, 500000 );

  token = strtok( NULL, delim );
  Interest_Rate = get_float( token );

  token = strtok( NULL, delim );
  Rate_Type = get_int( token, 0, 1 );

  token = strtok( NULL, ":/\n" );
  ARM_Principal = get_double( token, 1, 500000 );
  if( ARM_Principal < 31 ) {
    MM = static_cast<int>( ARM_Principal );
    if( Record_Type == 12 ) {
      ARM_Principal = Original_Value;
    } else {
      ARM_Principal = 0;
    }
  } else {
    token = strtok( NULL, "/\n" );
    MM = get_int( token, 1, 12 );
  }

  token = strtok( NULL, "/\n" );
  DD = get_int( token, 1, 31 );

  token = strtok( NULL, delim );
  YYYY = get_int( token, 0, 3000 );

  token = strtok( NULL, delim );
  Term = get_int( token, 1, 30 );

  token = strtok( NULL, delim );
  Loan_Type = get_int( token, 0, 1 );

  token = strtok( NULL, delim );
  Property_ID = get_double( token, 0, (pow( 10, 11 ) - 1) );

  token = strtok( NULL, delim );
  Loan_Status = get_int( token, 0, 2 );

  token = strtok( NULL, delim );
  for( a = 0; a < 4 && token; a++ ) {
    Borrower_ID[a] = get_double( token, 0, (pow( 10, 11 ) - 1) );

    token = strtok( NULL, delim );
    temp_name = get_name( token );
    if( temp_name ) {
       strcpy( Borrower_Name[a],  temp_name );
    }

    token = strtok( NULL, delim );
  }
}
